/*
 * FCKeditor - The text editor for Internet - http://www.fckeditor.net
 * Copyright (C) 2003-2008 Frederico Caldeira Knabben
 * 
 * == BEGIN LICENSE ==
 * 
 * Licensed under the terms of any of the following licenses at your
 * choice:
 * 
 *  - GNU General Public License Version 2 or later (the "GPL")
 *    http://www.gnu.org/licenses/gpl.html
 * 
 *  - GNU Lesser General Public License Version 2.1 or later (the "LGPL")
 *    http://www.gnu.org/licenses/lgpl.html
 * 
 *  - Mozilla Public License Version 1.1 or later (the "MPL")
 *    http://www.mozilla.org/MPL/MPL-1.1.html
 * 
 * == END LICENSE ==
 */

package net.fckeditor;

import javax.servlet.http.HttpServletRequest;

import net.fckeditor.handlers.PropertiesLoader;
import net.fckeditor.tool.Compatibility;
import net.fckeditor.tool.Utils;
import net.fckeditor.tool.XHtmlTagTool;

/**
 * Java representation of the FCKeditor. This class creates the HTML code for
 * the FCKeditor based on the following things:
 * <ul>
 * <li>browser capabilities</li>
 * <li>different properties settings managed by the {@link PropertiesLoader}</li>
 * <li>settings from the FCKeditor tag, template engines and other systems</li>
 * </ul>
 * 
 * @version $Id: FCKeditor.java 2765 2008-12-14 19:59:38Z mosipov $
 */
public class FCKeditor {

	private FCKeditorConfig config;
	private String instanceName;
	private String value = new String();
	private HttpServletRequest request;

	// defaults
	private String toolbarSet = PropertiesLoader.getEditorToolbarSet();
	private String width = PropertiesLoader.getEditorWidth();
	private String height = PropertiesLoader.getEditorHeight();
	private String basePath = PropertiesLoader.getEditorBasePath();

	/**
	 * Main constructor.<br />
	 * All important settings are done here and will be preset by the defaults
	 * taken from {@link PropertiesLoader}. Any parameter except instanceName
	 * failing {@link Utils#isNotBlank(String)} will be ignored.
	 * 
	 * @param request
	 *            request object
	 * @param instanceName
	 *            unique name
	 * @param width
	 *            width
	 * @param height
	 *            height
	 * @param toolbarSet
	 *            toolbarSet name
	 * @throws IllegalArgumentException
	 *             if instanceName is not valid HTML id
	 * @throws NullPointerException
	 *             if request is a null reference
	 */
	public FCKeditor(final HttpServletRequest request,
			final String instanceName, final String width, final String height,
			final String toolbarSet, final String value, final String basePath) {

		if (request == null)
			throw new NullPointerException("the request cannot be null");
		this.request = request;

		if (Utils.isBlank(instanceName))
			throw new IllegalArgumentException(
					"instanceName must be a valid HTML id");
		this.instanceName = instanceName;

		if (Utils.isNotBlank(width))
			this.width = width;
		if (Utils.isNotBlank(height))
			this.height = height;
		if (Utils.isNotBlank(toolbarSet))
			this.toolbarSet = toolbarSet;
		if (Utils.isNotBlank(value))
			this.value = value;
		if (Utils.isNotBlank(basePath))
			this.basePath = request.getContextPath().concat(basePath);
		else
			this.basePath = request.getContextPath().concat(this.basePath);

		config = new FCKeditorConfig();
	}

	/**
	 * Just a wrapper to {@link FCKeditor}.
	 * 
	 * @param request
	 *            request object
	 * @param instanceName
	 *            unique name
	 * @throws IllegalArgumentException
	 *             if instanceName is not valid HTML id
	 * @throws NullPointerException
	 *             if request is a null reference
	 */

	public FCKeditor(final HttpServletRequest request, final String instanceName) {
		this(request, instanceName, null, null, null, null, null);
	}

	/**
	 * Set the unique name of the editor
	 * 
	 * @param instanceName
	 *            name
	 */
	public void setInstanceName(final String instanceName) {
		this.instanceName = instanceName;
	}

	/**
	 * Set the initial value to be edited as HTML markup.
	 * 
	 * @param value
	 *            value
	 */
	public void setValue(final String value) {
		this.value = value;
	}

	/**
	 * Sets the directory where the FCKeditor resides on the server.<br />
	 * <strong>Remarks</strong>: Avoid using relative paths. Use an absolute
	 * path from the context (e.g. /fckeditor).
	 * 
	 * @param basePath
	 *            path
	 */
	public void setBasePath(final String basePath) {
		this.basePath = basePath;
	}

	/**
	 * Set the name of the toolbar to display
	 * 
	 * @param toolbarSet
	 *            toolbar name
	 */
	public void setToolbarSet(final String toolbarSet) {
		this.toolbarSet = toolbarSet;
	}

	/**
	 * Set the width of the textarea
	 * 
	 * @param width
	 *            width
	 */
	public void setWidth(final String width) {
		this.width = width;
	}

	/**
	 * Set the height of the textarea
	 * 
	 * @param height
	 *            height
	 */
	public void setHeight(final String height) {
		this.height = height;
	}

	/**
	 * Get the advanced configuration set.<br />
	 * By adding elements to this collection you can override the settings
	 * specified in the config.js file.
	 * 
	 * @return configuration collection
	 */
	public FCKeditorConfig getConfig() {
		return config;
	}

	/**
	 * Set the advanced configuration set.
	 * 
	 * @param config
	 *            configuration collection
	 */
	public void setConfig(FCKeditorConfig config) {
		this.config = config;
	}

	/**
	 * Escape base XML entities as specified <a
	 * href="http://en.wikipedia.org/wiki/Xml#Entity_references">here</a>.
	 * 
	 * @param str
	 *            Text to escape.
	 * @return Escaped text.
	 */
	private String escapeXml(String str) {

		StringBuffer sb = new StringBuffer();

		int len = str.length();
		char c;

		for (int i = 0; i < len; i++) {

			c = str.charAt(i);
			switch (c) {
			case '&':
				sb.append("&amp;");
				break;
			case '<':
				sb.append("&lt;");
				break;
			case '>':
				sb.append("&gt;");
				break;
			case '"':
				sb.append("&quot;");
				break;
			// XML actually defines &apos; as entity for the apostrophe but we
			// user rather the numerical reference to avoid XHTML 1.0 validation
			// problems
			case '\'':
				sb.append("&#39;");
				break;
			default:
				sb.append(c);
				break;
			}
		}

		return sb.toString();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see #createHtml()
	 */
	/**
	 * This method simply wraps to {@link #createHtml()}.
	 */
	@Override
	public String toString() {
		return createHtml();
	}

	/**
	 * Minimum implementation, see ticket #27 for detailed information. Generate
	 * the HTML Code for the editor.<br />
	 * Evaluate the browser capabilities and generate the editor if compatible,
	 * or a simple textarea otherwise.
	 * 
	 * @return FCKeditor HTML code
	 */
	public String createHtml() {
		StringBuffer strEditor = new StringBuffer();

		strEditor.append("<div>");
		String encodedValue = escapeXml(value);

		if (Compatibility.check(request.getHeader("user-agent"))) {
			strEditor.append(createInputForVariable(instanceName, instanceName,
					encodedValue));

			// create config HTML
			String configStr = config.getUrlParams();
			if (Utils.isNotEmpty(configStr))
				strEditor.append(createInputForVariable(null, instanceName
						.concat("___Config"), configStr));

			// create IFrame
			String sLink = basePath.concat(
					"/editor/fckeditor.html?InstanceName=")
					.concat(instanceName);
			if (Utils.isNotEmpty(toolbarSet))
				sLink += "&amp;Toolbar=".concat(toolbarSet);
			XHtmlTagTool iframeTag = new XHtmlTagTool("iframe",
					XHtmlTagTool.SPACE);
			iframeTag.addAttribute("id", instanceName.concat("___Frame"));
			iframeTag.addAttribute("src", sLink);
			iframeTag.addAttribute("width", width);
			iframeTag.addAttribute("height", height);
			iframeTag.addAttribute("frameborder", "0");
			iframeTag.addAttribute("scrolling", "no");
			strEditor.append(iframeTag);

		} else {
			XHtmlTagTool textareaTag = new XHtmlTagTool("textarea",
					encodedValue);
			textareaTag.addAttribute("name", instanceName);
			textareaTag.addAttribute("rows", "4");
			textareaTag.addAttribute("cols", "40");
			textareaTag.addAttribute("wrap", "virtual");
			textareaTag.addAttribute("style", "width: ".concat(width).concat(
					"; height: ").concat(height));
		}
		strEditor.append("</div>");
		return strEditor.toString();
	}

	/**
	 * Returns an input type="hidden" for the given attributes.
	 * 
	 * @param name
	 *            Name attribute of the input tag.
	 * @param id
	 *            Id attribute of the input tag.
	 * @param value
	 *            Value attribute of the input tag.
	 * @return The produced XHTML tag.
	 */
	private String createInputForVariable(final String name, final String id,
			final String value) {
		XHtmlTagTool tag = new XHtmlTagTool("input");
		if (Utils.isNotEmpty(id))
			tag.addAttribute("id", id);
		if (Utils.isNotEmpty(name))
			tag.addAttribute("name", name);
		tag.addAttribute("value", value);
		tag.addAttribute("type", "hidden");
		return tag.toString();
	}
}
